VERSION 5.00
Begin VB.Form frmExample 
   Caption         =   "Example Usage of DBNetwork"
   ClientHeight    =   9825
   ClientLeft      =   60
   ClientTop       =   450
   ClientWidth     =   10380
   LinkTopic       =   "Form1"
   ScaleHeight     =   10414.95
   ScaleMode       =   0  'User
   ScaleWidth      =   10380
   StartUpPosition =   1  'CenterOwner
End
Attribute VB_Name = "frmExample"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = False
Option Explicit

Const FRAMERATE = 60
Const GAME_WIDTH = 512
Const GAME_HEIGHT = 512

Const CHAR_TEX = 3
Const CHAR_WIDTH = 32
Const CHAR_HEIGHT = 32
Const CHAR_MOVEMENT_SPEED = 5

Const GRASS_WIDTH = 64
Const GRASS_HEIGHT = 32

Const MAX_PLAYERS = 4

Dim Engine As DBTurbo2DEngine
Dim bExit As Boolean, bRunning As Boolean

Private Sub Form_Load()

  Me.Show
  
  SetUpGame
  DoGameLoop
  
End Sub

Private Sub SetUpGame()
  Set Engine = New DBTurbo2DEngine
  
  With Engine
    .InitializeDisplay Me.hWnd, True, GAME_WIDTH, GAME_HEIGHT
    .SetMaxLevel 5
  End With
  
  Engine.DBFPS.SetFrameRate FRAMERATE
  
  Engine.SetUpMapView 0, 0, GAME_WIDTH, GAME_HEIGHT, 0
  
  With Engine.DBKeyInput
    .Initialize Me.hWnd
    .Add DIK_ESCAPE
    .Add DIK_LEFT
    .SetAutoFire 2, True
    .Add DIK_RIGHT
    .SetAutoFire 3, True
    .Add DIK_UP
    .SetAutoFire 4, True
    .Add DIK_DOWN
    .SetAutoFire 5, True
  End With
  
  With Engine.DBSound
    .Initialize Me.hWnd
    .SetSoundFolder App.Path & "/Sfx/"
    .Add "Text"
    .SetUpDirectionalSound 100, GAME_WIDTH, 10
  End With
  
  With Engine.DBTexture
    .SetFolder App.Path & "/Gfx/"
    .SetColorKeyBlue 255
    .SetColorKeyGreen 0
    .SetColorKeyRed 255
    .Add 1024, "isotiles", , 2
    .Add 512 'Map 1
    'Set colorkey to black
    .SetColorKeyBlue 0
    .SetColorKeyGreen 0
    .SetColorKeyRed 0
    .Add 128, "char", , 2
    .Add 512
  End With
  
  SetUpGrass 2, App.Path & "\map.txt"
  SetUpGrass 4, App.Path & "\map.txt"
  
  With Engine.DBMap
    'Map 1 - Grass
    .SetXCount 1, 1
    .SetYCount 1, 1
    .SetLooping 1, True
    .SetSubMapWidth 1, GAME_WIDTH
    .SetSubMapHeight 1, GAME_HEIGHT
    .SetVisible 1, True
    .SetXIncrement 1, CHAR_MOVEMENT_SPEED
    .SetYIncrement 1, CHAR_MOVEMENT_SPEED
    .SetZOrder 1, 0
    'Map 1 - Grass
    .SetXCount 2, 1
    .SetYCount 2, 1
    .SetLooping 2, True
    .SetSubMapWidth 2, GAME_WIDTH
    .SetSubMapHeight 2, GAME_HEIGHT
    .SetVisible 2, True
    .SetXReference 2, 1
    .SetYReference 2, 1
    .SetXIncrement 2, CHAR_MOVEMENT_SPEED
    .SetYIncrement 2, CHAR_MOVEMENT_SPEED
    .SetZOrder 2, 0
  End With
  
  'Set up submaps
  With Engine.DBSubMap
    'Map 1 - Grass
    .SetTextureReference 1, 1, 1, 2
    .SetTextureReference 2, 1, 1, 4
  End With
  
  'Set up auto move
  With Engine.DBAutoMove
    .SetOn True
    .SetMapReference 1
    .SetSpriteReference 1
    .SetWidth CHAR_WIDTH
    .SetHeight CHAR_HEIGHT
    .SetSWidth GAME_WIDTH / 2
    .SetSHeight GAME_HEIGHT / 2
  End With
  
  With Engine.DBText
    .CreateFont "ArialBlack", 16
    
    .Add 'Info text at the top
    .SetZOrder 1, 5
    .SetText 1, "Isometric Example"
    .QMSetPutRectangle 1, 0, 0, GAME_WIDTH, 30
    .SetAlpha 1, 255
    .Add 'Text showing FPS
    .QMSetPutRectangle 2, 0, .GetHeight(1), GAME_WIDTH, 30
    .SetAlpha 1, 200
    .SetZOrder 2, 5
  End With
  
  With Engine.DBSprite
    .Add
    .SetMapReference 1, 1
    .SetTextureReference 1, CHAR_TEX
    .SetXPosition 1, GAME_WIDTH / 2
    .SetYPosition 1, GAME_HEIGHT / 2
    .QMSetPutRectangle 1, -CHAR_WIDTH / 2, -CHAR_HEIGHT / 2, CHAR_WIDTH, CHAR_HEIGHT
    .QMSetGetRectangle 1, 0, 64, CHAR_WIDTH, CHAR_HEIGHT
    .SetZOrder 1, 2
    .SetVisible 1, True
  End With
End Sub

Private Sub DoGameLoop()
 
  bRunning = True
  
  Dim ix As Integer
  Do Until bExit = True
    With Engine
    
      .DBKeyInput.PollKeyBoard
      
      If .DBKeyInput.ReturnKeyDown(1) = True Then bExit = True

      If .DBKeyInput.ReturnKeyDown(2) = True Then
        .DBMath.SpriteIncrementLeft 1, CHAR_MOVEMENT_SPEED
      End If
      If .DBKeyInput.ReturnKeyDown(3) = True Then
        .DBMath.SpriteIncrementRight 1, CHAR_MOVEMENT_SPEED
      End If
      If .DBKeyInput.ReturnKeyDown(4) = True Then
        .DBMath.SpriteIncrementUp 1, CHAR_MOVEMENT_SPEED
      End If
      If .DBKeyInput.ReturnKeyDown(5) = True Then
        .DBMath.SpriteIncrementDown 1, CHAR_MOVEMENT_SPEED
      End If
      
      .Render
      
      .DBText.SetText 2, .DBFPS.GetFPS & " FPS"
    End With
    'DoEvents
  Loop
  
  bRunning = False
  
  EndApp
End Sub

Private Sub SetUpGrass(DestIndex As Integer, FilePath As String)

  Dim FileNum As Integer, XTiles() As String, YRowNum As Integer
  FileNum = FileSystem.FreeFile
  YRowNum = 0

  Open FilePath For Input As #FileNum

  Do Until EOF(FileNum) = True
    ReDim Preserve XTiles(YRowNum)
    Input #FileNum, XTiles(YRowNum)
    YRowNum = YRowNum + 1
  Loop

  Close FileNum
  
  On Error Resume Next
  'Loop to every 16th position on texture and blit the according water to it.
  Dim X As Single, Y As Single
  Dim GetX As Single, GetY As Single, PutX As Single, PutY As Single
  For Y = 0 To (GAME_HEIGHT - GRASS_HEIGHT) Step GRASS_HEIGHT
    For X = 0 To (GAME_WIDTH - GRASS_WIDTH) Step GRASS_WIDTH
      If Mid(XTiles(Y / GRASS_WIDTH), X / GRASS_HEIGHT + 1, 1) <> "-" Then
        GetX = Int(Mid(XTiles(Y / GRASS_WIDTH), X / GRASS_HEIGHT + 1, 1)) * GRASS_WIDTH
        GetY = 0
        PutX = 128 + X ' - (GRASS_WIDTH / 2) * (Y / GRASS_WIDTH)
        PutY = 128 + Y ' - (GRASS_HEIGHT / 2) * (X / GRASS_HEIGHT)
        If DestIndex = 2 Then ' HACK!
            Engine.DBTexture.CopyRegion 1, DestIndex, GetX, GetY, PutX, PutY, GRASS_WIDTH, GRASS_HEIGHT
        ElseIf DestIndex = 4 Then
            Engine.DBTexture.CopyRegion 1, DestIndex, GetX, GetY, PutX + (GRASS_WIDTH / 2), PutY - (GRASS_HEIGHT / 2), GRASS_WIDTH, GRASS_HEIGHT
        End If
      End If
    Next X
  Next Y
  
End Sub

Private Sub EndApp()
  Unload Me
  End
End Sub

Private Sub Form_Unload(Cancel As Integer)
  If bRunning = True Then
    Cancel = 1
    bExit = True
  Else
    EndApp
  End If
End Sub

